library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

--Takes input from ADC module, stores it in RAM, and outputs a word ~14 words before the one being evaluated.
--The intention it to allow viewing of the signal before the triggering event. "Pre-roll" if you will.
--This is module (3 of 4). Output in the form of ram_word_1 goes to the storage module where it is
--written into RAM if the threshold module has been triggered.

entity bram_simple_3 is
  port (
    clk              : in  std_logic;
    rst              : in  std_logic;
    ram_wr_en        : in  std_logic;  							--strobe from ADC; sample is ready      
    adc_word_final_3 : in  std_logic_vector(15 downto 0);   --input sample from adc
    ram_word_3       : out std_logic_vector(15 downto 0));  --output sample from bram

end bram_simple_3;

architecture syn of bram_simple_3 is
  type   ram_type_3 is array (15 downto 0) of std_logic_vector (15 downto 0);
  signal ram_3   : ram_type_3;
  signal wr_addr : std_logic_vector(3 downto 0);
  
begin
  
  process (clk, rst)
  begin

    if rst = '1' then
      wr_addr <= x"0";
    elsif (clk'event and clk = '1') then
      ram_word_3 <= ram_3(conv_integer(wr_addr+x"2"));	-- Minus 14 (the +2 wraps around)
      if ram_wr_en = '1' then										-- ADC signals that a new word is ready
        ram_3(conv_integer(wr_addr)) <= adc_word_final_3;-- Write incoming sample into RAM
        wr_addr                      <= wr_addr + 1;		-- Increment address
      end if;	
    end if;
  end process;
end syn;
