library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
library unisim;
use unisim.vcomponents.all;

entity top is
  port(
    --15MHz adc clock
    adc_clk_1 : out std_logic;
    adc_clk_2 : out std_logic;
    adc_clk_3 : out std_logic;
    adc_clk_4 : out std_logic;

	 --control signal for ADCs
    cs_1 : out std_logic;
    cs_2 : out std_logic;
    cs_3 : out std_logic;
    cs_4 : out std_logic;
	 
	 --bits from ADCs
    adc_pin_1              : in  std_logic;
    adc_pin_2              : in  std_logic;
    adc_pin_3              : in  std_logic;
    adc_pin_4              : in  std_logic;
	 
    --UM232H pins
	 clk, rst       : in    std_logic;
    rxf_lo, txe_lo : in    std_logic;
    data           : inout std_logic_vector(7 downto 0);
    oe_out, rd_out, wr_out : out std_logic); 
	 
end top;

architecture Behavioral of top is


  signal data_in : std_logic_vector(7 downto 0);

  signal rd_lo, oe_lo, wr_lo : std_logic;
  signal wr_en               : std_logic;

  signal data_out : std_logic_vector(7 downto 0);

  signal adc_word_final_1 : std_logic_vector(15 downto 0);
  signal adc_word_final_2 : std_logic_vector(15 downto 0);
  signal adc_word_final_3 : std_logic_vector(15 downto 0);
  signal adc_word_final_4 : std_logic_vector(15 downto 0);

  signal ram_wr_en : std_logic;

  signal wave_cnt_1 : std_logic_vector(1 downto 0);
  signal wave_cnt_2 : std_logic_vector(1 downto 0);
  signal wave_cnt_3 : std_logic_vector(1 downto 0);
  signal wave_cnt_4 : std_logic_vector(1 downto 0);

-- signals from threshold modules
  signal thresh_signal_1 : std_logic;
  signal thresh_signal_2 : std_logic;
  signal thresh_signal_3 : std_logic;
  signal thresh_signal_4 : std_logic;

  signal threshold_level_1 : std_logic_vector(11 downto 0);
  signal threshold_level_2 : std_logic_vector(11 downto 0);
  signal threshold_level_3 : std_logic_vector(11 downto 0);
  signal threshold_level_4 : std_logic_vector(11 downto 0);

  signal ram_full_1 : std_logic;
  signal ram_full_2 : std_logic;
  signal ram_full_3 : std_logic;
  signal ram_full_4 : std_logic;

  signal ram_word_1 : std_logic_vector(15 downto 0);
  signal ram_word_2 : std_logic_vector(15 downto 0);
  signal ram_word_3 : std_logic_vector(15 downto 0);
  signal ram_word_4 : std_logic_vector(15 downto 0);

  signal final_word_addr : std_logic_vector(15 downto 0);
  signal final_word_addr_end : std_logic_vector(15 downto 0);
  signal final_word_1    : std_logic_vector(7 downto 0);

  signal read_begin : std_logic;
  signal read_pause : std_logic;
  signal read_end   : std_logic;

  signal chan_1_begin : std_logic;
  signal chan_2_begin : std_logic;
  signal chan_3_begin : std_logic;
  signal chan_4_begin : std_logic;

  signal cs      : std_logic;
  signal adc_clk : std_logic;
--  signal tx_reset  : std_logic;  
  signal tx_reset_1: std_logic;
  signal tx_reset_2: std_logic;
  signal tx_reset_3: std_logic;
  signal tx_reset_4: std_logic;

  signal wr_addr_sto_out: std_logic_vector(15 downto 0);
  signal out_addr_out: std_logic_vector(15 downto 0);

  component usb
    port(
      --device pins
      clk      : in    std_logic;
      rst      : in    std_logic;
      rxf_lo   : in    std_logic;
      txe_lo   : in    std_logic;
      data     : inout std_logic_vector(7 downto 0);
      oe_lo    : out   std_logic;
      rd_lo    : out   std_logic;
      wr_lo    : out   std_logic;
      --internal ports
      data_out : in    std_logic_vector(7 downto 0);
      data_in  : out   std_logic_vector(7 downto 0);
      -- from protocol
      wr_en    : in    std_logic
      );
  end component;

  component adc
    port(
      clk              : in  std_logic;
		--bits from ADCs
      adc_pin_1        : in  std_logic;
      adc_pin_2        : in  std_logic;
      adc_pin_3        : in  std_logic;
      adc_pin_4        : in  std_logic;
      rst              : in  std_logic;
		--ADC control signal
      cs               : out std_logic;
      --words constructed from bits
		adc_word_final_1 : out std_logic_vector(15 downto 0);
      adc_word_final_2 : out std_logic_vector(15 downto 0);
      adc_word_final_3 : out std_logic_vector(15 downto 0);
      adc_word_final_4 : out std_logic_vector(15 downto 0);
		--signal when each word is constructed
      ram_wr_en        : out std_logic;
		--15MHz clock out to ADCs
      adc_clk          : out std_logic
      );
  end component;

	COMPONENT tx_rx
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		rxf_lo : IN std_logic;
		txe_lo : IN std_logic;
		wr_lo : IN std_logic;
		rd_lo : IN std_logic;          
		wr_en : OUT std_logic;
		data_in : IN std_logic_vector(7 downto 0);
		data_out : OUT std_logic_vector(7 downto 0);
		final_word_addr : OUT std_logic_vector(15 downto 0);
		final_word_addr_end : OUT std_logic_vector(15 downto 0);
		
		--from ADC
		ram_wr_en : IN std_logic;
		
		--from Storage Module
		--final output word 
		final_word_1 : IN std_logic_vector(7 downto 0);
		--number of waveforms currently stored 
		wave_cnt_1 : IN std_logic_vector(1 downto 0);
		wave_cnt_2 : IN std_logic_vector(1 downto 0);
		wave_cnt_3 : IN std_logic_vector(1 downto 0);
		wave_cnt_4 : IN std_logic_vector(1 downto 0);
		--RAM full flags
		ram_full_1 : IN std_logic;
		ram_full_2 : IN std_logic;
		ram_full_3 : IN std_logic;
		ram_full_4 : IN std_logic;
		--Control signals 
		read_pause : IN std_logic;
		read_end : IN std_logic;
		
		
--		ram_word_1 : IN std_logic_vector(15 downto 0);
--		adc_word_final_1 : IN std_logic_vector(15 downto 0);

--		thresh_signal_1 : IN std_logic;
--		thresh_signal_2 : IN std_logic;
--		thresh_signal_3 : IN std_logic;
--		thresh_signal_4 : IN std_logic;


		
		--to threshold modules
		threshold_level_1 : OUT std_logic_vector(11 downto 0);
		threshold_level_2 : OUT std_logic_vector(11 downto 0);
		threshold_level_3 : OUT std_logic_vector(11 downto 0);
		threshold_level_4 : OUT std_logic_vector(11 downto 0);
		chan_1_begin : OUT std_logic;
		chan_2_begin : OUT std_logic;
		chan_3_begin : OUT std_logic;
		chan_4_begin : OUT std_logic;
		
		--to storage module
		read_begin : OUT std_logic;
--		tx_reset : OUT std_logic;
		tx_reset_1 : OUT std_logic;
		tx_reset_2 : OUT std_logic;
		tx_reset_3 : OUT std_logic;
		tx_reset_4 : OUT std_logic
		);
	END COMPONENT;

  component bram_simple_1
    port(
      clk              : in  std_logic;
      rst              : in  std_logic;
      ram_wr_en        : in  std_logic;  --strobe from ADC; sample is ready      
      adc_word_final_1 : in  std_logic_vector(15 downto 0);  --input sample from adc
      ram_word_1       : out std_logic_vector(15 downto 0)  --output sample from bram
      );
  end component;

  component bram_simple_2
    port(
      clk              : in  std_logic;
      rst              : in  std_logic;
      ram_wr_en        : in  std_logic;  --strobe from ADC; sample is ready      
      adc_word_final_2 : in  std_logic_vector(15 downto 0);  --input sample from adc
      ram_word_2       : out std_logic_vector(15 downto 0)  --output sample from bram
      );
  end component;

  component bram_simple_3
    port(
      clk              : in  std_logic;
      rst              : in  std_logic;
      ram_wr_en        : in  std_logic;  --strobe from ADC; sample is ready      
      adc_word_final_3 : in  std_logic_vector(15 downto 0);  --input sample from adc
      ram_word_3       : out std_logic_vector(15 downto 0)  --output sample from bram
      );
  end component;

  component bram_simple_4
    port(
      clk              : in  std_logic;
      rst              : in  std_logic;
      ram_wr_en        : in  std_logic;  --strobe from ADC; sample is ready      
      adc_word_final_4 : in  std_logic_vector(15 downto 0);  --input sample from adc
      ram_word_4       : out std_logic_vector(15 downto 0)  --output sample from bram
      );
  end component;

  component threshold_simple_1
    port(
      clk               : in  std_logic;
      rst               : in  std_logic;
      threshold_level_1 : in  std_logic_vector(11 downto 0);
      adc_word_final_1  : in  std_logic_vector(15 downto 0);
      ram_wr_en         : in  std_logic;  --write enable from adc
      chan_1_begin      : in  std_logic;  --initial enable from tx_rx
		ram_full_1			: in  std_logic;
      thresh_signal_1   : out std_logic   --strobe when threshold is crossed
      );
  end component;

  component threshold_simple_2
    port(
      clk               : in  std_logic;
      rst               : in  std_logic;
      threshold_level_2 : in  std_logic_vector(11 downto 0);
      adc_word_final_2  : in  std_logic_vector(15 downto 0);
      ram_wr_en         : in  std_logic;  --write enable from adc
      chan_2_begin      : in  std_logic;  --initial enable from tx_rx
		ram_full_2			: in  std_logic;
      thresh_signal_2   : out std_logic   --strobe when threshold is crossed
      );
  end component;

  component threshold_simple_3
    port(
      clk               : in  std_logic;
      rst               : in  std_logic;
      threshold_level_3 : in  std_logic_vector(11 downto 0);
      adc_word_final_3  : in  std_logic_vector(15 downto 0);
      ram_wr_en         : in  std_logic;  --write enable from adc
      chan_3_begin      : in  std_logic;  --initial enable from tx_rx
		ram_full_3        : in  std_logic;
      thresh_signal_3   : out std_logic   --strobe when threshold is crossed
      );
  end component;

  component threshold_simple_4
    port(
      clk               : in  std_logic;
      rst               : in  std_logic;
      threshold_level_4 : in  std_logic_vector(11 downto 0);
      adc_word_final_4  : in  std_logic_vector(15 downto 0);
      ram_wr_en         : in  std_logic;  --write enable from adc
      chan_4_begin      : in  std_logic;  --initial enable from tx_rx
		ram_full_4        : in  std_logic;
      thresh_signal_4   : out std_logic   --strobe when threshold is crossed
      );
  end component;

	COMPONENT storage
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		ram_wr_en : IN std_logic;
		--threshold crossing command
		thresh_signal_1 : IN std_logic;
		thresh_signal_2 : IN std_logic;
		thresh_signal_3 : IN std_logic;
		thresh_signal_4 : IN std_logic;
		--word from RAM buffer
		ram_word_1 : IN std_logic_vector(15 downto 0);
		ram_word_2 : IN std_logic_vector(15 downto 0);
		ram_word_3 : IN std_logic_vector(15 downto 0);
		ram_word_4 : IN std_logic_vector(15 downto 0);
		--reset from tx_rx
		tx_reset_1 : IN std_logic;
		tx_reset_2 : IN std_logic;
		tx_reset_3 : IN std_logic;
		tx_reset_4 : IN std_logic;
		
		
		final_word_addr : IN std_logic_vector(15 downto 0);
		final_word_addr_end : IN std_logic_vector(15 downto 0);        
		--ram full flag to storage and threshold
		ram_full_1 : OUT std_logic;
		ram_full_2 : OUT std_logic;
		ram_full_3 : OUT std_logic;
		ram_full_4 : OUT std_logic;
		--wave count to tx_rx for status byte
		wave_cnt_1 : OUT std_logic_vector(1 downto 0);
		wave_cnt_2 : OUT std_logic_vector(1 downto 0);
		wave_cnt_3 : OUT std_logic_vector(1 downto 0);
		wave_cnt_4 : OUT std_logic_vector(1 downto 0);
		--control communication between storage and tx_rx
		read_begin : IN std_logic;
		read_pause : OUT std_logic;
		read_end : OUT std_logic;
		
		txe_lo: in std_logic;
		wr_lo: in std_logic;
		--current address storage writing to
		wr_addr_sto_out: OUT std_logic_vector(15 downto 0);
		--current address storage is reading from
		out_addr_out : out std_logic_vector(15 downto 0);
		--final output word going to tx_rx
		final_word_1 : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;
  
begin

  Inst_usb : usb port map(
    clk      => clk,
    rst      => rst,
	 --control pins from UM232H
    rxf_lo   => rxf_lo,
    txe_lo   => txe_lo,
	 --bidirectional data
    data     => data,
	 --control pins to UM232H
    oe_lo    => oe_lo,
    rd_lo    => rd_lo,
    wr_lo    => wr_lo,
    data_in  => data_in,
    data_out => data_out,
    wr_en    => wr_en
    );


  Inst_adc : adc port map(
    clk              => clk,
    adc_pin_1        => adc_pin_1,
    adc_pin_2        => adc_pin_2,
    adc_pin_3        => adc_pin_3,
    adc_pin_4        => adc_pin_4,
    cs               => cs,
    adc_word_final_1 => adc_word_final_1,
    adc_word_final_2 => adc_word_final_2,
    adc_word_final_3 => adc_word_final_3,
    adc_word_final_4 => adc_word_final_4,
    adc_clk          => adc_clk,
    ram_wr_en        => ram_wr_en,
    rst              => rst
    );

 Inst_tx_rx: tx_rx PORT MAP(
		clk => clk,
		rst => rst,
		rxf_lo => rxf_lo,
		txe_lo => txe_lo,
		data_in => data_in,
		wr_en => wr_en,
		data_out => data_out,
		ram_wr_en => ram_wr_en,
		final_word_1 => final_word_1,
		final_word_addr => final_word_addr,
		final_word_addr_end => final_word_addr_end,
		threshold_level_1 => threshold_level_1,
		threshold_level_2 => threshold_level_2,
		threshold_level_3 => threshold_level_3,
		threshold_level_4 => threshold_level_4,
		chan_1_begin => chan_1_begin,
		chan_2_begin => chan_2_begin,
		chan_3_begin => chan_3_begin,
		chan_4_begin => chan_4_begin,
		wave_cnt_1 => wave_cnt_1,
		wave_cnt_2 => wave_cnt_2,
		wave_cnt_3 => wave_cnt_3,
		wave_cnt_4 => wave_cnt_4,
		ram_full_1 => ram_full_1,
		ram_full_2 => ram_full_2,
		ram_full_3 => ram_full_3,
		ram_full_4 => ram_full_4,
--		ram_word_1 => ram_word_1,
--		adc_word_final_1 => adc_word_final_1,
		read_begin => read_begin,
		read_pause => read_pause,
		read_end => read_end,
--		tx_reset => tx_reset,
		tx_reset_1 => tx_reset_1,
		tx_reset_2 => tx_reset_2,
		tx_reset_3 => tx_reset_3,
		tx_reset_4 => tx_reset_4,
--		thresh_signal_1 => thresh_signal_1,
--		thresh_signal_2 => thresh_signal_2,
--		thresh_signal_3 => thresh_signal_3,
--		thresh_signal_4 => thresh_signal_4,
		wr_lo => wr_lo,
		rd_lo => rd_lo
	);


  Inst_bram_simple_1 : bram_simple_1 port map(
    clk              => clk,
    rst              => rst,
    ram_wr_en        => ram_wr_en,
    adc_word_final_1 => adc_word_final_1,
    ram_word_1       => ram_word_1
    );

  Inst_bram_simple_2 : bram_simple_2 port map(
    clk              => clk,
    rst              => rst,
    ram_wr_en        => ram_wr_en,
    adc_word_final_2 => adc_word_final_2,
    ram_word_2       => ram_word_2
    );

  Inst_bram_simple_3 : bram_simple_3 port map(
    clk              => clk,
    rst              => rst,
    ram_wr_en        => ram_wr_en,
    adc_word_final_3 => adc_word_final_3,
    ram_word_3       => ram_word_3
    );

  Inst_bram_simple_4 : bram_simple_4 port map(
    clk              => clk,
    rst              => rst,
    ram_wr_en        => ram_wr_en,
    adc_word_final_4 => adc_word_final_4,
    ram_word_4       => ram_word_4
    );

  Inst_threshold_simple_1 : threshold_simple_1 port map(
    clk               => clk,
    rst               => rst,
    threshold_level_1 => threshold_level_1,
    adc_word_final_1  => adc_word_final_1,
    ram_wr_en         => ram_wr_en,
    chan_1_begin      => chan_1_begin,
	 ram_full_1			 => ram_full_1,
    thresh_signal_1   => thresh_signal_1
    );

  Inst_threshold_simple_2 : threshold_simple_2 port map(
    clk               => clk,
    rst               => rst,
    threshold_level_2 => threshold_level_2,
    adc_word_final_2  => adc_word_final_2,
    ram_wr_en         => ram_wr_en,
    chan_2_begin      => chan_2_begin,
	 ram_full_2			 => ram_full_2,
    thresh_signal_2   => thresh_signal_2
    );

  Inst_threshold_simple_3 : threshold_simple_3 port map(
    clk               => clk,
    rst               => rst,
    threshold_level_3 => threshold_level_3,
    adc_word_final_3  => adc_word_final_3,
    ram_wr_en         => ram_wr_en,
    chan_3_begin      => chan_3_begin,
	 ram_full_3 		 => ram_full_3,
    thresh_signal_3   => thresh_signal_3
    );

  Inst_threshold_simple_4 : threshold_simple_4 port map(
    clk               => clk,
    rst               => rst,
    threshold_level_4 => threshold_level_4,
    adc_word_final_4  => adc_word_final_4,
    ram_wr_en         => ram_wr_en,
    chan_4_begin      => chan_4_begin,
	 ram_full_4 		 => ram_full_4,
    thresh_signal_4   => thresh_signal_4
    );

	Inst_storage: storage PORT MAP(
		clk => clk,
		rst => rst,
		ram_wr_en => ram_wr_en,
		thresh_signal_1 => thresh_signal_1,
		thresh_signal_2 => thresh_signal_2,
		thresh_signal_3 => thresh_signal_3,
		thresh_signal_4 => thresh_signal_4,
		ram_word_1 => ram_word_1,
		ram_word_2 => ram_word_2,
		ram_word_3 => ram_word_3,
		ram_word_4 => ram_word_4,
		ram_full_1 => ram_full_1,
		ram_full_2 => ram_full_2,
		ram_full_3 => ram_full_3,
		ram_full_4 => ram_full_4,
		tx_reset_1 => tx_reset_1,
		tx_reset_2 => tx_reset_2,
		tx_reset_3 => tx_reset_3,
		tx_reset_4 => tx_reset_4,
		wave_cnt_1 => wave_cnt_1,
		wave_cnt_2 => wave_cnt_2,
		wave_cnt_3 => wave_cnt_3,
		wave_cnt_4 => wave_cnt_4,
		read_begin => read_begin,
		read_pause => read_pause,
		txe_lo => txe_lo,
		wr_lo => wr_lo,
		wr_addr_sto_out => wr_addr_sto_out,
		out_addr_out => out_addr_out,
		read_end => read_end,
		final_word_addr => final_word_addr,
		final_word_addr_end => final_word_addr_end,
		final_word_1 => final_word_1
	);

  process (clk, rst, rd_lo, oe_lo, wr_lo, cs, adc_clk)
  begin

    rd_out    <= rd_lo;
    oe_out    <= oe_lo;
    wr_out    <= wr_lo;
    cs_1      <= cs;
    cs_2      <= cs;
    cs_3      <= cs;
    cs_4      <= cs;
    adc_clk_1 <= adc_clk;
    adc_clk_2 <= adc_clk;
    adc_clk_3 <= adc_clk;
    adc_clk_4 <= adc_clk;
    
  end process;
end Behavioral;
